#include <cstring>
#include "splitscreen.h"

SplitScreen::SplitScreen(GenericScreen *s1, GenericScreen *s2, GenericScreen *s3, GenericScreen *s4)
	: valid(false)
{
	subscreens[0] = s1;
	subscreens[1] = s2;
	subscreens[2] = s3;
	subscreens[3] = s4;

	memset(subbufs[0], 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);
	memset(subbufs[1], 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);
	memset(subbufs[2], 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);
	memset(subbufs[3], 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);
}

SplitScreen::~SplitScreen()
{
}

bool SplitScreen::check_invalidated()
{
	if (!valid)
		return true;
	
	for (unsigned i = 0; i < 4; ++i) {
		if (subscreens[i] && subscreens[i]->check_invalidated())
			return true;
	}

	return false;
}

void SplitScreen::draw(unsigned char *buf)
{
	for (unsigned i = 0; i < 4; ++i) {
		if (subscreens[i] && subscreens[i]->check_invalidated()) {
			subscreens[i]->draw(subbufs[i]);
		}
	}
	
	downscale_2x2(buf, subbufs[0]);
	downscale_2x2(buf + (SCREEN_WIDTH/2) * 4, subbufs[1]);
	downscale_2x2(buf + SCREEN_WIDTH * (SCREEN_HEIGHT/2) * 4, subbufs[2]);
	downscale_2x2(buf + SCREEN_WIDTH * (SCREEN_HEIGHT/2) * 4 + (SCREEN_WIDTH/2) * 4, subbufs[3]); 

	// make divider lines
	unsigned char *ptr = buf + (SCREEN_HEIGHT/2) * SCREEN_WIDTH * 4;
	for (unsigned x = 0; x < SCREEN_WIDTH; ++x) {
		*ptr++ = 255;
		*ptr++ = 255;
		*ptr++ = 255;
		*ptr++ = 0;
	}
	
	ptr = buf + (SCREEN_WIDTH/2) * 4;
	for (unsigned y = 0; y < SCREEN_HEIGHT; ++y) {
		ptr[0] = 255;
		ptr[1] = 255;
		ptr[2] = 255;
		ptr[3] = 0;

		ptr += SCREEN_WIDTH * 4;
	}
	
	valid = true;
}
	
// simple box filter (blah)
void SplitScreen::downscale_2x2(unsigned char *dst, unsigned char *src)
{
	for (unsigned y = 0; y < (SCREEN_HEIGHT/2); ++y) {
		unsigned char *sptr1 = src + (y*2) * SCREEN_WIDTH * 4;
		unsigned char *sptr2 = src + (y*2+1) * SCREEN_WIDTH * 4;
		unsigned char *dptr = dst + y * SCREEN_WIDTH * 4;
		
		for (unsigned x = 0; x < (SCREEN_WIDTH/2); ++x) {
			*dptr++ = (sptr1[0] + sptr1[4] + sptr2[0] + sptr2[4]) >> 2;  // red
			*dptr++ = (sptr1[1] + sptr1[5] + sptr2[1] + sptr2[5]) >> 2;  // green
			*dptr++ = (sptr1[2] + sptr1[6] + sptr2[2] + sptr2[6]) >> 2;  // blue
			*dptr++ = (sptr1[3] + sptr1[7] + sptr2[3] + sptr2[7]) >> 2;  // alpha

			sptr1 += 8;
			sptr2 += 8;
		}
	}
}

