#include <cstdio>
#include <algorithm>

#include "resolution.h"
#include "top10scorescreen.h"
#include "fonts.h"

Top10ScoreScreen::Top10ScoreScreen(pqxx::connection &conn, unsigned tournament)
	: conn(conn), tournament(tournament), scores_changed(conn, "scores"), valid(false)
{
}

Top10ScoreScreen::~Top10ScoreScreen()
{
}

bool Top10ScoreScreen::check_invalidated()
{
	if (!valid)
		return true;
	if (!scores_changed.get_flag())
		return false;

	// check that there are indeed changes, otherwise don't bother
	std::vector<TopScore> scores;
	conn.perform(FetchTopScoresForTournament(tournament, 10, &scores));
	
	for (std::vector<TopScore>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		if (seen_topscore.count(*i) == 0) {
			return true;
		}
	}
	
	return false;
}

void Top10ScoreScreen::draw(unsigned char *buf)
{
	scores_changed.reset_flag();
	memset(buf, 0, SCREEN_WIDTH * SCREEN_HEIGHT * 4);

	// fetch the top 10 scores
	std::vector<TopScore> scores;
	conn.perform(FetchTopScoresForTournament(tournament, 10, &scores));

	{
		unsigned width = my_draw_text("Today's top 10 scores", NULL, 40.0);
		my_draw_text("Today's top 10 scores", buf, 40.0, LOGICAL_SCREEN_WIDTH/2 - width/2, 60);
	}

	// simple headings
	my_draw_text("Player", buf, 12.0, 70, 100);
	my_draw_text("Song", buf, 12.0, 250, 100);
	my_draw_text("Score", buf, 12.0, 710, 100);
	
	unsigned row = 1, y = 140;
	for (std::vector<TopScore>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		char str[16];
		unsigned r = 255, g = 255, b = 255;

		// print new entries in red
		if (seen_topscore.count(*i) == 0 && seen_topscore.size() > 0) {
			g = b = 0;
		}

		std::sprintf(str, "%u", row++);
		unsigned width = my_draw_text(str, NULL, 24.0);
		my_draw_text(str, buf, 24.0, 30 - width/2, y);

		my_draw_text(i->nick, buf, 24.0, 70, y, r, g, b);

		if (my_draw_text(i->title, NULL, 24.0) > 430) {
			my_draw_text(i->shorttitle, buf, 24.0, 250, y, r, g, b);
		} else {
			my_draw_text(i->title, buf, 24.0, 250, y, r, g, b);
		}
		
		std::sprintf(str, "%u", i->score);
		width = my_draw_text(str, NULL, 24.0);
		my_draw_text(str, buf, 24.0, 728 - width/2, y, r, g, b);

		y += 40;
	}
	
	valid = true;
	
	seen_topscore.erase(seen_topscore.begin(), seen_topscore.end());
	std::copy(scores.begin(), scores.end(), std::inserter(seen_topscore, seen_topscore.end()));
}

int Top10ScoreScreen::get_priority()
{
	return 5;
}
